.submit-row a.cancel-link-nojs,
.submit-row a.cancel-link {
  display: block;
  background: #ba2121;
  border-radius: 4px;
  padding: 10px 15px;
  height: 15px;
  line-height: 15px;
  color: #fff;
}



.submit-row .cancel-btn {
  float: left;
  background: #ba2121 !important;
}

.submit-row .cancel-btn:focus,
.submit-row .cancel-btn:hover,
.submit-row .cancel-btn:active {
  background: #a41515 !important;
}

.submit-row a.cancel-link-nojs:focus,
.submit-row a.cancel-link-nojs:hover,
.submit-row a.cancel-link-nojs:active,
.submit-row a.cancel-link:focus,
.submit-row a.cancel-link:hover,
.submit-row a.cancel-link:active {
  background: #a41515;
}

.changed-data table {
  width: 100%;
}

.changed-data th,
.changed-data td {
  width: 30%;
  white-space: nowrap;
}

.hidden {
  display: none;
}
